'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.getStore = getStore;
exports.getStoreProps = getStoreProps;
exports.connectStore = connectStore;
exports.registerStore = registerStore;
exports.unregisterStore = unregisterStore;
exports.match = match;

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _Hook = require('./Hook');

var _Hook2 = _interopRequireDefault(_Hook);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var storeList = [];

function getStore(Store, getStoreHandler) {
  if (getStoreHandler) {
    return getStoreHandler(Store);
  }
  if (typeof Store.getStore === 'function') {
    return Store.getStore();
  }
  if (!Store.instanceWarp) {
    Store.instanceWarp = new Store();
  }
  return Store.instanceWarp;
}

function getStoreProps(matchNames, props, aliasNames) {
  var hooks = _Hook2.default.hooks[_Hook.HookTypes.connectStore] || [];
  var hookProps = {};
  if (!Array.isArray(matchNames)) {
    matchNames = [matchNames];
  }
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = (0, _getIterator3.default)(storeList), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var store = _step.value;

      var Store = void 0;
      var aliasName = void 0;
      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = (0, _getIterator3.default)(matchNames), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var matchItem = _step2.value;

          // 每一个matchItem也是一个数组
          if (match(store.storeNames, matchItem)) {
            if (store.filter) {
              if (store.filter(props)) {
                Store = store.Store;
              }
            } else {
              Store = store.Store;
            }
            // 支持自定义别名
            if (Store && aliasNames.length > matchNames.indexOf(matchItem)) {
              aliasName = aliasNames[matchNames.indexOf(matchItem)];
            }
            // 匹配过就跳出
            break;
          }
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      if (Store) {
        hookProps[aliasName || store.aliasName || store.storeNames.join('_')] = getStore(Store, store.getStoreHandler);
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  hooks.forEach(function (hook) {
    hookProps = (0, _extends3.default)({}, hookProps, hook(props));
  });
  return hookProps;
}

// matchName 支持 * 和 **
//   * 一级任意   ** 多级任意字符
function connectStore(matchName) {
  var args = arguments;
  return function (WarpComponent) {
    var StoreBindComponent = function StoreBindComponent(props) {
      var matchNames = [];
      // 支持自定义别名
      var aliasNames = [];
      if (args.length > 1) {
        // 支持多参数数组
        matchName = args;
      }
      // 支持三种参数形式
      if (typeof matchName === 'string') {
        if (matchName) {
          matchNames.push(matchName.split('.'));
          aliasNames.length += 1;
        }
        // 自动补全命名空间
        if (WarpComponent.__module) {
          matchNames.push(WarpComponent.__module.split('.').concat(matchName.split('.')));
          aliasNames.length += 1;
        }
      } else if (Array.isArray(matchName)) {
        matchNames = matchNames.concat(matchName.map(function (name) {
          return name.split('.');
        }));
        aliasNames.length += matchName.length;
      } else if ((typeof matchName === 'undefined' ? 'undefined' : (0, _typeof3.default)(matchName)) === 'object') {
        for (var aliasName in matchName) {
          var ClassName = matchName[aliasName];
          if (ClassName) {
            matchNames.push(ClassName.split('.'));
            aliasNames.push(aliasName);
          }
          // 自动补全命名空间
          if (WarpComponent.__module) {
            matchNames.push(WarpComponent.__module.split('.').concat(ClassName.split('.')));
            aliasNames.push(aliasName);
          }
        }
      }
      var storeProps = getStoreProps(matchNames, props, aliasNames);
      return _react2.default.createElement(
        WarpComponent,
        (0, _extends3.default)({}, storeProps, props),
        props.children
      );
    };
    StoreBindComponent.__module = WarpComponent.__module;
    StoreBindComponent.__version = WarpComponent.__version;
    return StoreBindComponent;
  };
}

function registerStore(storeName, aliasName, filter, getStoreHandler) {
  (0, _assert2.default)(storeName);
  (0, _assert2.default)(!filter || typeof filter === 'function');
  (0, _assert2.default)(!getStoreHandler || typeof getStoreHandler === 'function');

  return function (Store) {
    storeList.push({
      aliasName: aliasName || storeName,
      storeNames: storeName.split('.'),
      Store: Store,
      filter: filter,
      getStoreHandler: getStoreHandler
    });
  };
}

function unregisterStore(storeName) {
  var storeNames = storeName.split('.');
  var removes = storeList.filter(function (store) {
    return match(store.storeNames, storeNames);
  });
  var _iteratorNormalCompletion3 = true;
  var _didIteratorError3 = false;
  var _iteratorError3 = undefined;

  try {
    for (var _iterator3 = (0, _getIterator3.default)(removes), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
      var item = _step3.value;

      var index = storeList.indexOf(item);
      storeList.splice(index, 1);
    }
  } catch (err) {
    _didIteratorError3 = true;
    _iteratorError3 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion3 && _iterator3.return) {
        _iterator3.return();
      }
    } finally {
      if (_didIteratorError3) {
        throw _iteratorError3;
      }
    }
  }
}

function match(appNames, matchNames) {
  var matchIndex = 0;
  // **前匹配
  for (var i = 0; i < appNames.length && matchIndex < matchNames.length; i++) {
    if (appNames[i] !== matchNames[i] && matchNames[matchIndex] !== '*' && matchNames[matchIndex] !== '**') {
      return false;
    }
    if (matchNames[matchIndex] !== '**') {
      matchIndex++;
    }
  }
  // 匹配**后匹配
  for (var _i = appNames.length, _matchIndex = matchNames.length; _i > 0 && _matchIndex > 0; _i--) {
    if (appNames[_i] !== matchNames[_i] && matchNames[_matchIndex] !== '*' && matchNames[_matchIndex] !== '**') {
      return false;
    }
    if (matchNames[_matchIndex] !== '**') {
      _matchIndex--;
    }
  }
  return true;
}
//# sourceMappingURL=Store.js.map