'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Link = exports.Switch = exports.Route = exports.Router = exports.observer = exports.connect = exports.autobind = exports.connectStore = exports.connectStyle = exports.translate = exports.t = exports.serverStore = exports.messageStore = exports.routerStore = exports.i18nStore = exports.themeStore = exports.systemStore = exports.userStore = undefined;

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

exports.fetchJson = fetchJson;
exports.sendFetch = sendFetch;
exports.toQueryString = toQueryString;
exports.connectUrl = connectUrl;
exports.fetchServer = fetchServer;

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _reactNative = require('react-native');

var _coreDecorators = require('core-decorators');

var decorators = _interopRequireWildcard(_coreDecorators);

var _i18next = require('i18next');

var _i18next2 = _interopRequireDefault(_i18next);

var _User = require('../stores/User');

var _User2 = _interopRequireDefault(_User);

var _System = require('../stores/System');

var _System2 = _interopRequireDefault(_System);

var _Theme = require('../stores/Theme');

var _Theme2 = _interopRequireDefault(_Theme);

var _I18n = require('../stores/I18n');

var _I18n2 = _interopRequireDefault(_I18n);

var _Router = require('../stores/Router');

var _Router2 = _interopRequireDefault(_Router);

var _Message = require('../stores/Message');

var _Message2 = _interopRequireDefault(_Message);

var _Server = require('../stores/Server');

var _Server2 = _interopRequireDefault(_Server);

var _LocalStore = require('./LocalStore');

var _LocalStore2 = _interopRequireDefault(_LocalStore);

var _internal = require('./internal');

var internal = _interopRequireWildcard(_internal);

var _Theme3 = require('../core/Theme');

var theme = _interopRequireWildcard(_Theme3);

var _I18n3 = require('../core/I18n');

var i18n = _interopRequireWildcard(_I18n3);

var _Store = require('../core/Store');

var Store = _interopRequireWildcard(_Store);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// ************** stores *********************

var userStore = exports.userStore = _User2.default.getStore(); // 用户参数
var systemStore = exports.systemStore = _System2.default.getStore(); // 系统参数
var themeStore = exports.themeStore = _Theme2.default.getStore(); // 主题
var i18nStore = exports.i18nStore = _I18n2.default.getStore(); // 多语言
var routerStore = exports.routerStore = _Router2.default.getStore(); // 路由
var messageStore = exports.messageStore = _Message2.default.getStore(); // 消息服务
var serverStore = exports.serverStore = _Server2.default.getStore(); // 服务器

// *************** local && theme ******************

var t = exports.t = _i18next2.default.tx;

var translate = exports.translate = i18n.translate;
var connectStyle = exports.connectStyle = theme.connectStyle;
var connectStore = exports.connectStore = Store.connectStore;

var autobind = exports.autobind = decorators.autobind;

var connect = exports.connect = function connect(local, theme, store) {
  var obser = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : true;
  var atob = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : true;

  var storeObj = void 0;
  if (typeof store === 'string') {
    storeObj = { appStore: store };
  } else {
    storeObj = store;
  }
  return function (WarppedComponent) {
    (0, _assert2.default)(WarppedComponent);
    var ConnectedComponent = WarppedComponent;
    if (atob) {
      autobind(ConnectedComponent);
    }
    if (obser) {
      ConnectedComponent = observer(ConnectedComponent);
    }
    if (store) {
      ConnectedComponent = connectStore(storeObj)(ConnectedComponent);
    }
    if (theme) {
      ConnectedComponent = connectStyle(theme)(ConnectedComponent);
    }
    if (local) {
      ConnectedComponent = translate(local)(ConnectedComponent);
    }
    return ConnectedComponent;
  };
};

// ******************* mobx ********************

var observer = exports.observer = _reactNative.Platform.OS === 'web' ? require('mobx-react').observer : require('mobx-react/native').observer;

// ******************* router ******************************

var RouterInternal = void 0,
    RouteInternal = void 0,
    SwitchInternal = void 0,
    LinkInternal = void 0;
switch (_reactNative.Platform.OS) {
  case 'android':
  case 'ios':
  case 'windows':
  case 'macos':
    {
      var NativeRouter = require('react-router-native');
      RouterInternal = NativeRouter.NativeRouter;
      RouteInternal = NativeRouter.Route;
      SwitchInternal = NativeRouter.Switch;
      LinkInternal = NativeRouter.Link;
      break;
    }
  case 'web':
    {
      var BrowserRouter = require('react-router-dom');
      RouterInternal = BrowserRouter.BrowserRouter;
      RouteInternal = BrowserRouter.Route;
      SwitchInternal = BrowserRouter.Switch;
      LinkInternal = BrowserRouter.Link;
      break;
    }
  default:
    console.error(tx('不支持的路由平台'), _reactNative.Platform.OS);
    break;
}

var Router = exports.Router = RouterInternal;
var Route = exports.Route = RouteInternal;
var Switch = exports.Switch = SwitchInternal;
var Link = exports.Link = LinkInternal;

// ********************** fetch utils ******************************

var ntCacheName = 'ntcache@';

function loadFromCache(user, url) {
  return new _promise2.default(function (resolve, reject) {
    _LocalStore2.default.load({
      key: ntCacheName + (user || 'guest'),
      id: url,
      autoSync: false
    }).then(function (ret) {
      resolve(ret);
    }).catch(function (err) {
      // 如果没有找到数据且没有sync方法， 或者有其他异常，则在catch中返回
      switch (err.name) {
        case 'NotFoundError':
          resolve(null);
          break;
        case 'ExpiredError':
          resolve(null);
          break;
        default:
          console.warn(err.message);
          reject(internal.tx('缓存加载失败'));
      }
    });
  });
}

function saveToCache(user, url, data) {
  _LocalStore2.default.save({
    key: ntCacheName + (user || 'guest'),
    id: url,
    rawData: data,
    // 如果不指定过期时间，则会使用defaultExpires参数 如果设为null，则永不过期
    expires: null
  });
}

// 是否启用fetch缓存的全局设置
global.enableFetchCache = true;

function fetchJson(_ref) {
  var url = _ref.url,
      options = _ref.options,
      _ref$withCache = _ref.withCache,
      withCache = _ref$withCache === undefined ? true : _ref$withCache,
      _ref$withToken = _ref.withToken,
      withToken = _ref$withToken === undefined ? true : _ref$withToken;

  return sendFetch({ url: url, options: options, withCache: withCache, withToken: withToken, repContextType: 'json' });
}

function sendFetch(_ref2) {
  var url = _ref2.url,
      options = _ref2.options,
      _ref2$withCache = _ref2.withCache,
      withCache = _ref2$withCache === undefined ? true : _ref2$withCache,
      _ref2$withToken = _ref2.withToken,
      withToken = _ref2$withToken === undefined ? true : _ref2$withToken,
      _ref2$repContextType = _ref2.repContextType,
      repContextType = _ref2$repContextType === undefined ? 'json' : _ref2$repContextType;

  (0, _assert2.default)(url);
  // 如果没有网络，尝试从缓存中加载数据
  if (!global.isConnected && withCache && global.enableFetchCache) {
    return loadFromCache(_User2.default.getStore().getToken(), url);
  }
  return new _promise2.default(function (resolve, reject) {
    var optionsWithToken = (0, _extends3.default)({}, options);
    if (withToken) {
      optionsWithToken.headers = (0, _extends3.default)({
        sptoken: _User2.default.getStore().getToken()
      }, optionsWithToken.headers);
    }
    fetch(url, optionsWithToken).then(function (response) {
      if (!response.ok) {
        throw new Error(internal.tx('网络连接失败') + ' (' + response.status + ')');
      }
      if (repContextType === 'json') {
        return response.text().then(function (text) {
          if (!text) {
            return {};
          }
          return JSON.parse(text);
        });
      } else {
        return response.text();
      }
    }).then(function (responseJson) {
      if (repContextType) {
        if (!responseJson.errno) {
          if (withCache) {
            saveToCache(_User2.default.getStore().getToken(), url, responseJson.data);
          }
          resolve(responseJson.data);
        } else {
          console.log(internal.tx('网络连接失败'));
          console.warn(responseJson.errmsg);
          reject(responseJson.errmsg);
        }
      } else {
        resolve(responseJson);
      }
    }).catch(function (err) {
      console.log(internal.tx('网络连接失败'), url);
      console.error(err);
      reject(internal.tx('网络连接失败'));
    });
  });
}

function toQueryString(obj) {
  return obj ? (0, _keys2.default)(obj).sort().map(function (key) {
    var val = obj[key];
    if (Array.isArray(val)) {
      return val.sort().map(function (val2) {
        return encodeURIComponent(key) + '=' + encodeURIComponent(val2);
      }).join('&');
    }
    return encodeURIComponent(key) + '=' + encodeURIComponent(val);
  }).join('&') : '';
}

function connectUrl(url, path, args) {
  (0, _assert2.default)(url);
  (0, _assert2.default)(path);
  var fullurl = url.trimRight('/') + '/' + path.trimLeft('/');
  if (args) {
    var qs = toQueryString(args);
    if (qs) {
      fullurl = fullurl.trimRight('?').trimRight('&');
      if (fullurl.indexOf('?') > -1) {
        fullurl += '&' + qs;
      } else {
        fullurl += '?' + qs;
      }
    }
  }
  return fullurl;
}

// fetch with user token header
function fetchServer(_ref3) {
  var url = _ref3.url,
      headers = _ref3.headers,
      data = _ref3.data,
      _ref3$method = _ref3.method,
      method = _ref3$method === undefined ? 'get' : _ref3$method,
      _ref3$withCache = _ref3.withCache,
      withCache = _ref3$withCache === undefined ? true : _ref3$withCache,
      _ref3$withToken = _ref3.withToken,
      withToken = _ref3$withToken === undefined ? true : _ref3$withToken,
      _ref3$repContextType = _ref3.repContextType,
      repContextType = _ref3$repContextType === undefined ? 'json' : _ref3$repContextType;

  (0, _assert2.default)(url);
  var server = _Server2.default.getStore().currentServer;

  if (!server || !server.address) {
    throw internal.tx('服务器无法连接');
  }

  return sendFetch({
    url: connectUrl(server.address, url, method === 'get' ? data : ''),
    options: {
      method: method,
      headers: (0, _extends3.default)({
        'Content-Type': 'application/json'
      }, headers),
      body: method === 'post' ? toQueryString(data) : undefined
    },
    withCache: withCache,
    withToken: withToken,
    repContextType: repContextType
  });
}

// export function fetchPlatform({
//   url,
//   headers,
//   data,
//   method = 'get',
//   withCache = true,
//   withToken = true,
//   repContextType = 'json'
// }) {
//   assert(url);
//
//   return sendFetch({
//     url: connectUrl(config.platform.address, url, (method === 'get'
//       ? data
//       : '')),
//     options: {
//       method,
//       headers: {
//         'Content-Type': 'application/json',
//         ...headers
//       },
//       body: method === 'post'
//         ? toQueryString(data)
//         : undefined
//     },
//     withCache,
//     withToken,
//     repContextType
//   });
// }
//# sourceMappingURL=helper.js.map